

%FILE: RF2VSapp2.m
%UPDATE BHJ2008MAY: Modified to compute VSapp from rotated receiver functions
%PURPOSE: 
%  Compute several values of apparent VS from 
%  corresponing Z and R receiver functions.
%  According to equations (5) and (6) in 
%  L. Svenningsen, B. H. Jacobsen (2007): 
%    Absolute S-velocity estimation from receiver functions 
%    Geophysical Journal International 170 (3), 10891094. 
%    doi:10.1111/j.1365-246X.2006.03505.x
%  i_app=atan(R/Z); VSapp=sin(i_app/2)/slowness;
%INPUTS: 
%  Q_rf: The deconvolved Q-componet
%  L_rf: The deconvolved L-component
%  ts: Accociated sampling times
%  Delta [Degrees]: Epicenter distance of event
%  hypocenterdepth [km]: Depth of hypocenter
%  Ts : Radii of averaging before computation of values of VS_app
%  rot_ang [deg] : rotation angle used in computation of Q and L.
%                  Defaults to rot_ang=0, i.e. no rotation
%  tpf: Taperfraction; tpf = 0 in  
%              Z(i) = sum(Zs.*W(Ts(i))) 
%              where W is a cos-taper of radius Ts(i) 
%              with a flat top of radius tpf*Ts(i) 
%              W(t) = 1 for abs(t)<tpf*Ts(i), 
%              W(t) = 0 for abs(t)>Ts(i),
%              W(t) = 0.5*(1+cos(2*pi*(t-tpf*Ts(i))/(Ts(i)*(1-tpf))));
%OUTPUTS:
%  VSapps [km/s]: A vector of apparent VS, one value for each Ts(i)
%  slowness [s/degree]: The corresponding slowness used in the computations, 
%                       computed from P-arrival times in IASPEI-model. 
%SUBFUNCTIONS AND FILES:
% Calls function get_tt internally
% get_tt reads P.mat which contains arrival times of P-wave
%CALL:
%[VSapps,slowness] = RF2VSapp2(Q_rf,L_rf,ts,Delta,hypocenterdepth,Ts,rot_ang,tpf)
function [VSapps,slowness] = ...
    RF2VSapp2(Q_rf,L_rf,ts,Delta,hypocenterdepth,Ts,rot_ang,tpf)
L_rf = L_rf(:); Q_rf = Q_rf(:); ts = ts(:);
if nargin<8
    tpf=0; %W just ordinary cosine bell
end
if nargin<7
    rot_ang = 0; %defaults to no rotation  
end
if nargin<6
    Ts = 10.^(-0.5:0.1:1.5); %0.3 second < Ts < 32 seconds 
end
if nargin<5
    hypocenterdepth=30; %km
end
if nargin<4
    error('you must specify Zs, Rs, ts and Delta')
end
if (Delta<30)|(Delta>100)
    %at present, we do not like these distances 
    VSapp=NaN;
else
    %compute the values
    [tP,slowness]=get_tt('P',Delta,hypocenterdepth);
    p = slowness / 111.111; %111.111 km per degree
    for i=1:length(Ts)
        i_nz = find(abs(ts)<Ts(i));
        i_nz2 = find(abs(ts)<=(tpf*Ts(i))); %the plateau
        ts(i_nz2) = ts(max(i_nz2));
        W = 0.5*(1 + cos(pi*(ts(i_nz)-tpf*Ts(i))/(Ts(i)*(1-tpf))));
       
        L = sum(L_rf(i_nz).*W);
        Q = sum(Q_rf(i_nz).*W);
        di_app = atan(Q/L); %rotation relative to rot_ang
        i_app = rot_ang*pi/180 + di_app;
        VSapps(i) = sin(i_app/2)/p; %km/s
    end
end

function [tt_out,p_est]=get_tt(phase,delt_wish,z_wish)
% call : [tt_wish,p_est]=get_tt(phase,delt_wish,z_wish)
% function to call locsat phase tables and interpolate traveltimes (cubic spline) 
% p is estimated from simple numerical derivative
% Make sure that P.MAT is available
str=['load ',phase,'.mat']; %depths, deg and tt!
eval(str);
diff_step = 1e-5; %degrees
for i=1:length(delt_wish)
    clear delts tt_wish
    delts=[delt_wish(i),delt_wish(i)+diff_step,delt_wish(i)-diff_step];
    zs=[z_wish(i),z_wish(i),z_wish(i)];
    if length(depths)<3
         tt_wish = interp2(depths',deg,tt,zs,delts,'spline');
     else
        tt_wish = interp2(depths',deg,tt,zs,delts,'cubic');
    end
    p_est(i)=(tt_wish(2)-tt_wish(3))/(2*diff_step);
    tt_out(i)=tt_wish(1);
end
tt_out=tt_out(:);
p_est=p_est(:);
