%FILE: moddistdept2RF.m
%PURPOSE: Compute synthetic receiver function with Gaussian wavelet for
%given hypocenter and given model
%INPUTS:
%dist [deg]: Epicenter distance in degrees
%depth [km]: Depth of hypocenter in km
%model : Specification of model, 
%              either as a single integer pointing to a predefined model
%              or as a structure: model=struct('zs',zs,'VPs',VPs);
%        where
%              model.zs [km]: Depths to bottoms of layers
%              model.VPs [km/s]: Interval P-velocities
%              model.VSs [km/s]: Optional S-velosities (default to VPs/sqrt(3))
%              model.rhos [kg/m3]: Optional densities (default to Birch medium)
%ts [s]: Sampling of forward computations (note: is updated to ts_nice)
%Gaussian_factor : As defined in Ammon et al.
%plot_on : ==0: No plot; ==1: some plots
%OUTPUTS:
%r_rf : Radial receiver function, smoothed by input parameter "Gaussian_factor"
%up_rf : Corresponding vertical receiver function. Positive for upward displacement
%r : Radial response for Gaussian input at bottom of model
%up : Corresponding vertical displacement
%ts_nice : Time axis for r_rf etc. Has been extended so that a nicely factored fft results
%lump : A structure with a number of other possibly usefull intermediate results.
%CALL:
%        [r_rf,up_rf,r,up,ts_nice,lump]=moddistdept2RF(dist,depth,model,ts,Gaussian_factor,plot_on)
function [r_rf,up_rf,r,up,ts_nice,lump]=moddistdept2RF(dist,depth,model,ts,Gaussian_factor,plot_on)
if nargin<6
    plot_on=0;
end
if nargin<5
   Gaussian_factor = 2.5; %as used in Ammon et al
end    

if nargin<4
   ts = -5:0.05:30;
end

if nargin<3
    model = 1;
end

if nargin<2
    depth = 30; %standard hypocenter depth
end

if nargin<1
    dist = 70; %70 degrees ~ 7777 km
end

if isstruct(model)
    %model must have the fields zs and VPs
    zs = model.zs;
    VPs = model.VPs;
    if isfield(model,'VSs')
        VSs = model.VSs;
    else
        %if model.VSs is not specified then Poisson medium is assumed
        VSs = VPs/sqrt(3);
    end
    if isfield(model,'rhos')
        rhos = model.rhos;
    else
        %if model.rhos is not specified then Birch medium is assumed
        rhos = 320*VPs + 770; 
    end

else %model must be an integer pointing to specific models below
    model_number = model;
    switch model_number
    case 0, %homegeneous crust
        zs = [20,35];
        VPs = [5.8,5.8,5.8];
        VSs = VPs/sqrt(3);
        rhos = 320*VPs + 770; %Birch medium
    case 1, %IASPEI like
        zs = [20,35];
        VPs = [5.8,6.5,8.04];
        VSs = VPs/sqrt(3);
        rhos = 320*VPs + 770; %Birch medium
    case 2, % Gradient Moho
        N_gr = 2; %N_gr=0 means simple discontinuity
        L_gr = 5; %width of gradient
        zs_gr = linspace(-L_gr/2,L_gr/2,2*N_gr+1);
        zs = [20,35+zs_gr];
        VP_gr = linspace(6.5,8.04,2*N_gr+2);
        VPs = [5.8,6.5,VP_gr(2:end-1),8.04];
        VSs = VPs/sqrt(3);
        rhos = 320*VPs + 770; %Birch medium
    case 3, % IASPEI with inhomogeneities
        n_sub=3;
        dz = 5/n_sub;
        zs = [dz:dz:50];
        VPs = [5.8*ones(1,4*n_sub),6.5*ones(1,3*n_sub),8.04*ones(1,3*n_sub+1)];
        dVP = 0.1; %perturbation scatter
        randn('seed',1234) 
        dVPs = dVP*randn(size(VPs));
        VPs = VPs + dVPs;
        VSs = VPs/sqrt(3);
        rhos = 320*VPs + 770; %Birch medium
    case 4, % simple sediment model (naive VSs and rhos)
        zs = [3,20,35];
        VPs = [2,5.8,6.5,8.04]; %North Sea Cenozoic clay on bedrock
        VSs = VPs/sqrt(3);
        rhos = 320*VPs + 770; %Birch medium
        %Note that receiver function now differs a lot from radial
        %displacement (due to the deconvolution of the P-multiple)

    case 5, % Play with VP/VS in sediment
        zs  = [0.5,1.0,3.0,20 ,35 ];
        VPs = [1.8,2.2,3.5,5.8,6.5,8.04]; %Similar to FUR
        VPVSs=[2.5,1.9,1.8,1.8,1.8,1.8];
        VSs = VPs./VPVSs;
        rhos = 320*VPs + 770; %Birch medium
        %Note that receiver function now differs a lot from radial
        %displacement (due to the deconvolution of the P-multiple)
    case 6, % Ice
        zs = [3,50];
        VPs = [3.7,6.5,8.04]; %Similar to SUMG
        VPVSs=[2.2,1.8,1.8];
        VSs = VPs./VPVSs;
        rhos = [900,320*VPs(2:end) + 770]; %Birch medium
    otherwise, %IASPEI like model==1
        zs = [20,35];
        VPs = [5.8,6.5,8.04];
        VSs = VPs/sqrt(3);
        rhos = 320*VPs + 770; %Birch medium
    end %switch
end %else

%compute slowness from dist and depth
[tP,slowness]=get_tt('P',dist,depth);
pdeg = slowness; %assuming 111.111 km per degree

[r_rf,up_rf,r,up,ts_nice,lump]=rf_forward(zs,VPs,VSs,rhos,pdeg,ts,Gaussian_factor);
lump.pdeg=pdeg;
lump.tP = tP;
lump.zs = zs;
lump.VPs = VPs;
lump.VSs = VSs;

%plotting
if plot_on
    figure(1)
    stair_z([0,zs,max(zs)+5],VPs,'.-k')
    hold on
    stair_z([0,zs,max(zs)+5],VSs,'.-r')
    hold on
    stair_z([0,zs,max(zs)+5],rhos/1000,'.-b')
    title('VP=black, VS=red, Density=blue')
    axis auto

    figure(2)
    subplot(2,1,1);
    plot(ts_nice,r+0.5,'red',ts_nice,up,'k');
    hold on
    title('Radial displacement = red. Upward displacement = black')
    subplot(2,1,2);
    plot(ts_nice,r_rf+0.2,'red',ts_nice,up_rf,'k');
    hold on
    title('Radial rf = red. Upward rf = black')
end %if plot_on