%FILE: rf_forward.m
% Computation of receiver functions and displacements at the surface
% for plane P-wave impingent from below.
% The stack of homogeneous, isotropic layers is completed by "elastic air"
% rho = 1.2 kg/m3
% VP  = 0.34 km/s
% VS  = 0.02 km/s i.e. VERY low shear modulus
%INPUTS:
% zs [km]     : Surface at z=0, 
%               zs(1)   = Bottom of first layer
%               zs(end) = Top of halfspace
% VPs [km(s]  : VPs(1) between 0 and zs(1)
%               VPs(i) between zs(i-1) and zs(i)
%               VPs(end) below zs(end)
%               hence length(VPs) = length(zs)+1
% VSs [km/s]  : default VSs = VPs/sqrt(3)
% rhos [kg/m3]: default rhos = 0.32*VPs + 770; %Birch medium
% p [s/deg]   : wave slowness, defualt 6.4 s/deg
% ts [s]      : Time axis, assumed periodic. Take care of wrap around
% Gaussian_factor : With omegas = 2*pi*frequency the waveform spectrum is
%               H(omegas) = k*exp(-omegas.^2/(4*Gaussian_factor^2))
% OUTPUTS:
% r_rf : Radial reciever function
% up_rf: Receiver function for vertical component (positive upwards)
% r    : Radial component for unit impulse impingent
% up   : Vertical component for unit impulse impingent
% ts_nice: time samples, modified to get efficient fft.
% lump : Structure containting all sorts of intermediate results
% CALL:
%  [r_rf,up_rf,r,up,ts_nice,lump]=rf_forward(zs,VPs,VSs,rhos,p,ts,Gaussian_factor);

function [r_rf,up_rf,r,up,ts_nice,lump]=...
    rf_forward(zs,VPs,VSs,rhos,p,ts,Gaussian_factor);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Take care of default input parameters
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
if nargin<2
    if nargin<1
        zs = [20,35]; %IASPEI layers in crust
        VPs = [5.8,6.5,8.04]; %IASPEI velocities
    else
        error('Both zs and VPs must be specified')
    end
end
if nargin<3
    VSs = VPs/sqrt(3); %Poisson medium is default
end
if nargin<4
    rhos = 0.32*VPs + 770; %Birch medium
end
if nargin<5
    p=6.4; %ray slowness; seconds per degree
end
if nargin<6
    ts = -10:0.05:50;
end
if nargin<7
    Gaussian_factor = 2.5; %see Ammon 1991(?)
    %H(omegas) = k*exp(-omegas.^2/(4*Gaussian_factor^2));
end
if nargin<8
    nice_fft = 0;
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute surface response in Fourier domain
% for plane impingent delta P-wave wavefront
% Transmitted P-wave impulse arrives at surface at ts=0
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[fs,ts_nice,R,UP,lump]=rf_allmult_fd_3(VPs,VSs,rhos,zs,p,ts);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Compute the time domain displacements
% as well as the receiver function
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
omegas = 2*pi*fs(:);
H = exp(-omegas.^2/(4*Gaussian_factor^2));
h = real(ifft(H));
k = 1/max(h);
H = k*H; %normalized to amplitude 1.0 in time domain

r = real(ifft(H.*lump.R_fd));
up = real(ifft(H.*lump.UP_fd));

shift_fd = shift(fs(:),-min(ts_nice));
r_rf = real(ifft(H.*lump.R_fd./lump.UP_fd.*shift_fd)); %note that the spectrum of UP normally has no zero crossings
up_rf = real(ifft(H.*1.0.*shift_fd)); %UP./UP = 1.0

lump.H = H;
lump.h = h;
function [fs,ts_fft,R,UP,lump]=rf_allmult_fd_3(VPs_kms,VSs_kms,rhos_kg_m3,zs_km,p_sdeg,ts);
% Input units are km, km/s and s/deg and s
% Coversion to m, m/s, s/m  (and force all to be rows)

%p_sm = p_sdeg(:)'*180/(6371000*pi); 
p_sm = p_sdeg(:)'/111111; %1 degree per 111111 meter 
VPs = 1000*VPs_kms(:)'; 
VSs = 1000*VSs_kms(:)'; 
zs = 1000*zs_km(:)'; 
ts = ts(:)'; 
rhos = rhos_kg_m3(:)';

ns = 2;

%Complete the model as Poisson-Birch medium with air halfspace
%so that the bottom of the first layer is the earth surface at z(1). 
%Thus we take topography into account in a natural way.
%Insert free surface at z=0:
zs = [0,zs(:)']; %soil at sea level implicit; force a row
% Insert Air halfspace above free surface:
air_fac = 0.1; %air factor; 1=sea level; 0.1=stratosphere
% Density is assumed to obey Birch's-relation:

%rhos = (0.32*VPs + 770); %kg/m3 for VPs in m/s
VPs  = [340*air_fac,VPs];
rhos = [1.2*air_fac,rhos];
VSs  = [0.1*VPs(1)/sqrt(3),VSs]; %Slightly unphysical elastic air
   
N_layers = length(VPs)-2; %the number of finite layers, typically layers inside the crust

%Times
dt=ts(2)-ts(1);
Nt = length(ts);
T_max = ts(Nt);
N_fft = nice_fft(Nt); %pick next mixed power of 2 and 3
ts_fft = T_max - dt*((N_fft-1):-1:0); %pad before zero
T_min_fft = ts_fft(1);
df = 1/(N_fft*dt);
fs = df*[0:N_fft/2-1,-N_fft/2:-1];
%............................. now compute all conversions
[Td,Ru,Tu,Rd,S]=vrho2tr(p_sm,VPs,VSs,rhos);
for i=1:N_layers+1
    invTd=inv(Td(:,:,i));
    M(:,:,i) = [invTd,-invTd*Ru(:,:,i);Rd(:,:,i)*invTd,(Tu(:,:,i)-Rd(:,:,i)*invTd*Ru(:,:,i))];
end
                                                                                                                                                                                                                                                                                                                                                                                                                                                                                                        
%Wave inputs:
Ad = zeros(2,N_fft); %downgoing wavefields above earth surface; no input from above.
%Bu = [ones(1,N_fft);zeros(1,N_fft)]; %upgoing wavefields below lowermost interface; impulsive P-wave at t=T_min_fft and no S-wave
P_shift = sum(sqrt(1./VPs(2:N_layers+1).^2-p_sm^2).*diff(zs)); %apparent vertical traveltime of direct P-wave from bottom to surface.
Bu = [shift(fs,-T_min_fft - P_shift);zeros(1,N_fft)]; %upgoing wavefields below lowermost interface; impulsive P-wave at t=T_min_fft and no S-wave
%Wave responses (just prealocations:
Au = 9999*sqrt(-1)*ones(2,N_fft); %upgoing wavefields above earth surface
Bd = 9999*sqrt(-1)*ones(2,N_fft); %downgoing wavefields below lowermost interface
B_intern = zeros(N_fft,4,length(ns)); %accumulate Fouriertransforms of wavefields beneath indexed interfaces. 

C = zeros(4,4,N_layers+1); %delay effects, one for each homogeneous region
C(1,1,1) = sqrt(-1); %just to force the array size to be complex
%Kan vi njes med N_fft/2+1???
for j=1:N_fft/2+1
%for j=1:N_fft
  C(1,1,2:N_layers+1) = exp(+sqrt(-1)*2*pi*fs(j)*sqrt(1./VPs(2:N_layers+1).^2-p_sm^2).*diff(zs));  
  C(2,2,2:N_layers+1) = exp(+sqrt(-1)*2*pi*fs(j)*sqrt(1./VSs(2:N_layers+1).^2-p_sm^2).*diff(zs));  
  % disse to kan spares (se ende-kommentar)
  %C(3,3,2:N_layers+1) = exp(-sqrt(-1)*2*pi*fs(j)*sqrt(1./VPs(2:N_layers+1).^2-p_sm^2).*diff(zs)); %
  C(3,3,:) = conj(C(1,1,:));  
  %C(4,4,2:N_layers+1) = exp(-sqrt(-1)*2*pi*fs(j)*sqrt(1./VSs(2:N_layers+1).^2-p_sm^2).*diff(zs)); %
  C(4,4,:) = conj(C(2,2,:));   
  FBCM = sqrt(-1)*ones(4,4,N_layers+2);
  FBCM(:,:,N_layers+2)=eye(4);
  for i=N_layers+1:-1:2
      FBCM(:,:,i)=C(:,:,i)*M(:,:,i)*FBCM(:,:,i+1);
  end
  FBCM(:,:,1)=M(:,:,1)*FBCM(:,:,2);
  D = FBCM(:,:,1);
  Ddd = D(1:2,1:2);
  Dud = D(1:2,3:4);
  Ddu = D(3:4,1:2);
  Duu = D(3:4,3:4);
  
  warning('off')
  invDdd = inv(Ddd);
  %Bd(:,j) = invDdd*(Ad(:,j)-Dud*Bu(:,j));
  Bd(:,j) = Ddd\(Ad(:,j)-Dud*Bu(:,j));
  warning('on')
  %Au(:,j) = Ddu*inv(Ddd)*Ad(:,j) + (Duu-Ddu*inv(Ddd)*Dud)*Bu(:,j));
  Au(:,j) = Ddu*Bd(:,j) + Duu*Bu(:,j);
  B_bottom = [Bd(:,j);Bu(:,j)];
  for in=1:length(ns)
     %Fejl!:B_intern(j,:,in) = FBCM(:,:,in)*B_bottom;
     B_intern(j,:,in) = FBCM(:,:,ns(in))*B_bottom;
  end
end
% Now consider the remaining negative frequencies:
for k=N_fft/2+2:N_fft
    j=N_fft-k+2;
    Bd(:,k) = conj(Bd(:,j));
    Au(:,k) = conj(Au(:,j));
    %B_bottom = [Bd(:,k);Bu(:,k)];
    for in=1:length(ns)
         %B_intern(k,:,in) = FBCM(:,:,in)*B_bottom;
         B_intern(k,:,in) = conj(B_intern(j,:,in));
    end
end

Pu = Au(1,:); %P-wave in elastic air
Su = Au(2,:); %S-wave in elastic air

%--->>> BHJ change May 2008:
%VPs(in) changed to VPs(ns(in)) etc.
%effect is that previously we converted P and S below surface with 
% a PS2rz computed in air. The effect was small; a few percent! 
%This error is not present in code used for inversion, hence results in
%BSSA-paper "Enhanced ..." are valid.
for in=1:length(ns)
   Bpd2rz(:,in) = [VPs(ns(in))*p_sm;+sqrt(1-(VPs(ns(in))*p_sm)^2)];
   Bsd2rz(:,in) = [+sqrt(1-(VSs(ns(in))*p_sm)^2);-VSs(ns(in))*p_sm];
   Bpu2rz(:,in) = [VPs(ns(in))*p_sm;-sqrt(1-(VPs(ns(in))*p_sm)^2)];
   Bsu2rz(:,in) = [+sqrt(1-(VSs(ns(in))*p_sm)^2);VSs(ns(in))*p_sm];
   PS2rz(:,:,in) = [Bpd2rz(:,in),Bsd2rz(:,in),Bpu2rz(:,in),Bsu2rz(:,in)];
end

%Compute displacements R(fs) and UP(fs) beneath surface and associated receiver function RF(fs)
in = 1; %ns(1) should be 2 in order to make values relate to the position just below the surface
R_pd =  Bpd2rz(1,in).*B_intern(:,1,in);
UP_pd = -Bpd2rz(2,in).*B_intern(:,1,in);

R_sd =  Bsd2rz(1,in).*B_intern(:,2,in);
UP_sd = -Bsd2rz(2,in).*B_intern(:,2,in);

R_pu =  Bpu2rz(1,in).*B_intern(:,3,in);
UP_pu = -Bpu2rz(2,in).*B_intern(:,3,in);

R_su =  Bsu2rz(1,in).*B_intern(:,4,in);
UP_su = -Bsu2rz(2,in).*B_intern(:,4,in);

R  = R_pd + R_sd + R_pu + R_su;
UP = UP_pd + UP_sd + UP_pu + UP_su;
%T  = zeros(size(R));
lump.RF_fd = R./UP;
lump.R_fd = R;
lump.UP_fd = UP;

lump.FBCM = FBCM;
lump.B_bottom = B_bottom;
lump.B_intern = B_intern;
lump.PS2rz = PS2rz;
% Transform back to timedomain:
R = real(ifft(R));
UP = real(ifft(UP));

i_ns2 = find(ns==2);
if isempty(i_ns2); 
    error('ns must contain 2 to make it possible to compute displacements below surface'); 
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% file = vrho2tr.m
% Derived from ps_conv.m (update: 17/12-2002; after AkiRich.m 5/11-02 BHJ & Sonny Krag)
% Purpose: returns conversioncoefficients at several boundaries 
% S: structure with 16 fields, each representing a conversion type at boundary n 
%              [PdPu  SdPu  PuPu  SuPu ]
%		       [PdSu  SdSu  PuSu  SuSu ]
%         S =  [PdPd  SdPd  PuPd  SuPd ] = [Rd Tu]
%			   [PdSd  SdSd  PuSd  SuSd ]   [Td Ru]
% PdPu: a matrix of classical P reflection coefficients for incidence from above.
%       indexed as PdPu(ips,ins)
% :
% SuSd: a vector of S-wave refelction coefficients for incidence from below.
% Transmission of down-going P and S above interface, Ad = [AdP;AdS] 
% into down-going P and S below interface, Bd = [BdP;BdS] = Td*Ad, where
% Td = [PdPd SdPd ; PdSd SdSd];
% Likewise 
% Au = Ru*Ad; Ru = [PdPu SdPu;PdSu SdSu];
% Au = Tu*Bu; Tu = [PuPu SuPu;PuSu SuSu];
% Bd = Ru*Bu; Ru = [PuPd SuPd;PuSd SuSd];
% INPUTS:
% ns : boundary numbers for which to compute the conversions 
% p  : ray parameter sin(incidence angle)/alpha [s/m]
% alpha : P-velocities in layers [m/s]
% beta  : S-hastighed for alle lag [m/s]
% rho :  Densities in layers [kg/m3]
% OUTPUT: 
% Td = [PdPd SdPd ; PdSd SdSd] = S(3:4,1:2)
% Ru = [PdPu SdPu ; PdSu SdSu] = S(1:2,1:2)  
% Tu = [PuPu SuPu ; PuSu SuSu] = S(1:2,3:4)
% Rd = [PuPd SuPd ; PuSd SuSd] = S(2:4,3:4)
%
% CALL:  [Td,Ru,Tu,Ru,S]=vrho2tr(p,alpha,beta,rho)
function [Td,Ru,Tu,Rd,S]=vrho2tr(p,alpha,beta,rho)
for n=1:length(alpha)-1
   
  N_lay = length(alpha);
  if (n>=N_lay)|(n<0)
  error(['n illegal. Number of interfaces is',num2str(length(alpha))])
  end
   
  %p      =  ps(ip);
  alpha1  =  alpha(n);
  alpha2  =  alpha(n+1);
  beta1  =  beta(n);
  beta2  =  beta(n+1);
  rho1   =  rho(n);
  rho2   =  rho(n+1);

  i1  =  asin(p*alpha1);
  j1  =  asin(p*beta1);
  i2  =  asin(p*alpha2);
  j2  =  asin(p*beta2);

  M(1,1) = -alpha1*p;
  M(1,2) = -cos(j1);
  M(1,3) = alpha2*p;
  M(1,4) = cos(j2);

  M(2,1) = cos(i1);
  M(2,2) = -beta1*p;
  M(2,3) = cos(i2);
  M(2,4) = -beta2*p;


  M(3,1) = 2*rho1*beta1^2*p*cos(i1);
  M(3,2) = rho1*beta1*(1-2*beta1^2*p^2);
  M(3,3) = 2*rho2*beta2^2*p*cos(i2);
  M(3,4) = rho2*beta2*(1-2*beta2^2*p^2);

  M(4,1) = -rho1*alpha1*(1-2*beta1^2*p^2);
  M(4,2) = 2*rho1*beta1^2*p*cos(j1);
  M(4,3) = rho2*alpha2*(1-2*beta2^2*p^2);
  M(4,4) = -2*rho2*beta2^2*p*cos(j2);


  N(1,1) = alpha1*p;
  N(1,2) = cos(j1);
  N(1,3) = -alpha2*p;
  N(1,4) = -cos(j2);

  N(2,1) = cos(i1);
  N(2,2) = -beta1*p;
  N(2,3) = cos(i2);
  N(2,4) = -beta2*p;

  N(3,1) = 2*rho1*beta1^2*p*cos(i1);
  N(3,2) = rho1*beta1*(1-2*beta1^2*p^2);
  N(3,3) = 2*rho2*beta2^2*p*cos(i2);
  N(3,4) = rho2*beta2*(1-2*beta2^2*p^2);

  N(4,1) = rho1*alpha1*(1-2*beta1^2*p^2);
  N(4,2) = -2*rho1*beta1^2*p*cos(j1);
  N(4,3) = -rho2*alpha2*(1-2*beta2^2*p^2);
  N(4,4) = 2*rho2*beta2^2*p*cos(j2);


  A(:,:,n)   =  inv(M)*N;
  Td(:,:,n)  = A(3:4,1:2,n);
  Rd(:,:,n)  = A(1:2,1:2,n);  
  Tu(:,:,n)  = A(1:2,3:4,n);
  Ru(:,:,n)  = A(3:4,3:4,n);
  if nargout>4
    ip=1; %only one ray parameter
    S.PdPu(ip,n) = A(1,1);
    S.SdPu(ip,n) = A(1,2);
    S.PuPu(ip,n) = A(1,3);
    S.SuPu(ip,n) = A(1,4);
    S.PdSu(ip,n) = A(2,1);
    S.SdSu(ip,n) = A(2,2);
    S.PuSu(ip,n) = A(2,3);
    S.SuSu(ip,n) = A(2,4);
    S.PdPd(ip,n) = A(3,1);
    S.SdPd(ip,n) = A(3,2);
    S.PuPd(ip,n) = A(3,3);
    S.SuPd(ip,n) = A(3,4);
    S.PdSd(ip,n) = A(4,1);
    S.SdSd(ip,n) = A(4,2);
    S.PuSd(ip,n) = A(4,3);
    S.SuSd(ip,n) = A(4,4);
  end
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function exp_fac=shift(fs,t_shift);
N=length(fs);
exp_fac=exp(-i*2*pi*fs*t_shift);
exp_fac(N/2+1)=real(exp_fac(N/2+1));
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function nice_N = nice_fft(N)
N2=nextpow2(N);
N3=round(N2/1.5);
p2 = 2.^(1:N2);%we demand nice_N to be even
p3 = 3.^(0:N3);
[p22,p33]=meshgrid(p2,p3);
p2233 = p22.*p33;
p2233sort = sort(p2233(:));
nice_N = min(p2233sort(find(p2233sort>=N)));
